/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <ds/conf.h>
#include <ds/json.h>
#include <boost/variant.hpp>
#include "http_client.h"
#include "model/apartment.h"
#include "vdc.h"

namespace dss {

struct PutResource {
  ds::JsonFragment value;

  PutResource(const PutResource&) = delete;
  PutResource(PutResource&&) = default;
  PutResource(ds::JsonFragment&& value) : value(std::move(value)){};
};
struct DeleteResource {};
struct PatchResource {
  ds::JsonFragment diff;

  PatchResource(const PatchResource&) = delete;
  PatchResource(PatchResource&&) = default;
  PatchResource(ds::JsonFragment&& diff) : diff(std::move(diff)){};
};

typedef boost::variant<PutResource, DeleteResource, PatchResource> ResourceNotification;

struct CollectionNotification {
  std::string id;
  ResourceNotification notification;
};

BOOST_STRONG_TYPEDEF(std::string, BackendVdcId);
std::ostream& operator<<(std::ostream& ostream, const BackendVdcId& x);

BOOST_STRONG_TYPEDEF(std::string, BackendVdcDeviceId);
std::ostream& operator<<(std::ostream& ostream, const BackendVdcDeviceId& x);

class BackendVdcs {
public:
  virtual ~BackendVdcs();

  virtual void notify(const std::string& path, ResourceNotification&&) = 0;
  virtual StructureModifyingBusInterface& structureModifyingBusInterface() = 0;

  virtual BackendVdcId vdcIdForDeviceDsuid(const dsuid_t&) const = 0;
  virtual std::string vdcDsuidForId(const BackendVdcId& vdcId) const = 0;

  // Calls http request with added base url, authentication and language headers
  virtual ds::JsonFragment http(HttpRequest&& x, const boost::optional<std::locale>& = boost::none) const = 0;
};

std::unique_ptr<BackendVdcs> makeBackendVdcs(Apartment&, PropertySystem&, EventInterpreter&, ds::Conf&);

} // namespace dss
