#pragma once
#include "businterface.h"
#include "ds485/dsbusinterface.h"
#include "messages/vdc-messages.pb.h"

namespace dss {

class DSS;
class ModelMaintenance;
class Bus : public BusInterface, public StructureModifyingBusInterface {
public:
  Bus(DSS& dss, ModelMaintenance& modelMaintenance);

  DSBusInterface& dsBus() { return m_ds485Bus; }
  StructureModifyingBusInterface& dsmModifyBus(const dsuid_t& dsmId);

  DeviceBusInterface* getDeviceBusInterface() DS_OVERRIDE { return m_ds485Bus.getDeviceBusInterface(); }
  StructureQueryBusInterface* getStructureQueryBusInterface() DS_OVERRIDE {
    return m_ds485Bus.getStructureQueryBusInterface();
  }
  MeteringBusInterface* getMeteringBusInterface() DS_OVERRIDE { return m_ds485Bus.getMeteringBusInterface(); }
  StructureModifyingBusInterface* getStructureModifyingBusInterface() DS_OVERRIDE { return this; }
  ActionRequestInterface* getActionRequestInterface() DS_OVERRIDE { return m_ds485Bus.getActionRequestInterface(); }
  void setBusEventSink(BusEventSink* x) DS_OVERRIDE { return m_ds485Bus.setBusEventSink(x); }
  const std::string getConnectionURI() DS_OVERRIDE { return m_ds485Bus.getConnectionURI(); }
  void add(StructureModifyingBusInterface& x) DS_OVERRIDE { m_modifies.push_back(std::ref(x)); }

  std::string name() const DS_OVERRIDE;

  // StructureModifyingBusInterface
  void setZoneID(const dsuid_t& dsm, const devid_t device, const int zone) DS_OVERRIDE {
    dsmModifyBus(dsm).setZoneID(dsm, device, zone);
  }
  void createZone(const dsuid_t& dsm, const int zone) DS_OVERRIDE { dsmModifyBus(dsm).createZone(dsm, zone); }
  void removeZone(const dsuid_t& dsm, const int zone) DS_OVERRIDE { dsmModifyBus(dsm).removeZone(dsm, zone); }
  void addToGroup(const dsuid_t& dsm, const int group, const int device) DS_OVERRIDE {
    dsmModifyBus(dsm).addToGroup(dsm, group, device);
  }
  void removeFromGroup(const dsuid_t& dsm, const int group, const int device) DS_OVERRIDE {
    dsmModifyBus(dsm).removeFromGroup(dsm, group, device);
  }
  void removeDeviceFromDSMeter(const dsuid_t& dsm, const int device) DS_OVERRIDE {
    dsmModifyBus(dsm).removeDeviceFromDSMeter(dsm, device);
  }
  void removeDeviceFromDSMeters(const dsuid_t& device) DS_OVERRIDE;
  void sceneSetName(uint16_t zone, uint8_t group, uint8_t scene, const std::string& name) DS_OVERRIDE;
  void deviceSetName(dsuid_t dsm, devid_t device, const std::string& name) DS_OVERRIDE {
    dsmModifyBus(dsm).deviceSetName(dsm, device, name);
  }
  void meterSetName(dsuid_t dsm, const std::string& name) DS_OVERRIDE { dsmModifyBus(dsm).meterSetName(dsm, name); }
  void createGroup(uint16_t zone, uint8_t group, ApplicationType application, uint32_t applicationConfig,
      const std::string& name) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) {
      modify.get().createGroup(zone, group, application, applicationConfig, name);
    }
  }
  void removeGroup(uint16_t zone, uint8_t group) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().removeGroup(zone, group); }
  }
  void groupSetApplication(
      uint16_t zone, uint8_t group, ApplicationType application, uint32_t applicationConfig) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) {
      modify.get().groupSetApplication(zone, group, application, applicationConfig);
    }
  }
  void groupSetName(uint16_t zone, uint8_t group, const std::string& name) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().groupSetName(zone, group, name); }
  }
  void createCluster(
      uint8_t group, ApplicationType application, uint32_t applicationConfig, const std::string& name) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) {
      modify.get().createCluster(group, application, applicationConfig, name);
    }
  }
  void removeCluster(uint8_t cluster) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().removeCluster(cluster); }
  }
  void clusterSetName(uint8_t cluster, const std::string& name) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().clusterSetName(cluster, name); }
  }
  void clusterSetApplication(uint8_t cluster, ApplicationType application, uint32_t applicationConfig) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) {
      modify.get().clusterSetApplication(cluster, application, applicationConfig);
    }
  }
  void clusterSetProperties(uint8_t cluster, uint16_t location, uint16_t floor, uint16_t protectionClass) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) {
      modify.get().clusterSetProperties(cluster, location, floor, protectionClass);
    }
  }
  void clusterSetLockedScenes(uint8_t cluster, const std::vector<int>& lockedScenes) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().clusterSetLockedScenes(cluster, lockedScenes); }
  }
  void clusterSetConfigurationLock(uint8_t cluster, bool lock) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().clusterSetConfigurationLock(cluster, lock); }
  }
  void clusterSetOperationLock(uint8_t cluster, bool lock) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().clusterSetOperationLock(cluster, lock); }
  }
  void setButtonSetsLocalPriority(const dsuid_t& dsm, const devid_t device, bool setsPriority) DS_OVERRIDE {
    dsmModifyBus(dsm).setButtonSetsLocalPriority(dsm, device, setsPriority);
  }
  void setButtonCallsPresent(const dsuid_t& dsm, const devid_t device, bool callsPresent) DS_OVERRIDE {
    dsmModifyBus(dsm).setButtonCallsPresent(dsm, device, callsPresent);
  }
  void setZoneHeatingConfig(const dsuid_t& dsm, const uint16_t zone, const ZoneHeatingConfigSpec_t spec) DS_OVERRIDE {
    dsmModifyBus(dsm).setZoneHeatingConfig(dsm, zone, spec);
  }
  void setZoneHeatingOperationModes(
      const dsuid_t& dsm, const uint16_t zone, const ZoneHeatingOperationModeSpec_t spec) DS_OVERRIDE {
    dsmModifyBus(dsm).setZoneHeatingOperationModes(dsm, zone, spec);
  }
  void setZoneSensor(const uint16_t zone, SensorType sensorType, const dsuid_t& sensorDSUID) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().setZoneSensor(zone, sensorType, sensorDSUID); }
  }
  void resetZoneSensor(const uint16_t zone, SensorType sensorType) DS_OVERRIDE {
    BOOST_FOREACH (const auto& modify, m_modifies) { modify.get().resetZoneSensor(zone, sensorType); }
  }
  void setCircuitPowerStateConfig(
      const dsuid_t& dsm, const int index, const int setThreshold, const int resetThreshold) DS_OVERRIDE {
    dsmModifyBus(dsm).setCircuitPowerStateConfig(dsm, index, setThreshold, resetThreshold);
  }
  void setProperty(const dsuid_t& dsm,
      const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& properties) DS_OVERRIDE {
    dsmModifyBus(dsm).setProperty(dsm, properties);
  }
  vdcapi::Message getProperty(
      const dsuid_t& dsm, const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& query) DS_OVERRIDE {
    return dsmModifyBus(dsm).getProperty(dsm, query);
  }
  void setFireMuteEnabled(dsuid_t dsm, bool x) DS_OVERRIDE { return dsmModifyBus(dsm).setFireMuteEnabled(dsm, x); }
  void removeVdc(const dsuid_t& vdsmID, const dsuid_t& vdcID) DS_OVERRIDE { return dsmModifyBus(vdsmID).removeVdc(vdsmID, vdcID); }

private:
  DSS& m_dss;
  DSBusInterface m_ds485Bus;
  StructureModifyingBusInterface& m_ds485Modify;
  std::vector<std::reference_wrapper<StructureModifyingBusInterface>> m_modifies;
};

} // namespace dss
