/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "watchdog.h"
#include <ds/asio/catch.h>
#include <ds/log.h>
#include <ds/memory.h>
#include <boost/asio.hpp>
#include <tuple>

static const char* TAGS = "[dsAsioWatchdog][dsAsio][ds]";

TEST_CASE("dsAsioFileDescriptorsWatchdog", TAGS) {
  ds::asio::catch_::IoService ioService;
  auto&& LATENCY = ds::asio::catch_::LATENCY;
  ds::asio::FileDescriptorsWatchdog watchdog(ioService, LATENCY);

  SECTION("can be created and destroyed") {}

  SECTION("Fires when all file descriptors are consumed.") {
    std::vector<std::unique_ptr<boost::asio::local::stream_protocol::socket>> sockets;
    bool isOutOfFileDescriptors = false;
    try {
      for (int i = 0; i < 1000000; i++) {
        auto socket1 = ds::make_unique<boost::asio::local::stream_protocol::socket>(ioService);
        auto socket2 = ds::make_unique<boost::asio::local::stream_protocol::socket>(ioService);
        boost::asio::local::connect_pair(*socket1, *socket2);
        sockets.push_back(std::move(socket1));
        sockets.push_back(std::move(socket2));
      }
    } catch (const std::exception& e) {
      isOutOfFileDescriptors = true;
      // Create sockets until it fails
    }
    REQUIRE(isOutOfFileDescriptors);
    watchdog.setFire([&](const std::exception& e) {
      ioService.stop();
    });
    CHECK_STOP_RUN(ioService);
  }

  SECTION("Does not fire again when file descriptors are available.") { CHECK_NO_STOP_RUN(ioService); }
}
