/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "conf.h"
#include <ds/catch/catch.h>
#include <ds/log.h>
#include <boost/optional/optional_io.hpp>

static const char* TAGS = "[dsConf][ds]";

namespace {

TEST_CASE("dsConf", TAGS) {
  auto system = ds::makeSystem({
      .sysconfdir = ds::str(DS_SRC_DIR "/test/etc"),    //
      .datadir = ds::str(DS_SRC_DIR "/test/usr/share"), //
  });
  auto conf = ds::makeConf(*system);
  SECTION("tryRead reads etc only") { CHECK(conf->tryRead<int>("number-etc") == 1); }

  SECTION("tryRead reads usr only") { CHECK(conf->tryRead<int>("number-usr") == 2); }

  SECTION("tryRead reads etc before usr") { CHECK(conf->tryRead<int>("number-etc-usr") == 1); }

  SECTION("tryRead returns none without etc and usr") { CHECK(conf->tryRead<int>("number") == boost::none); }

  SECTION("readOr reads etc only") { CHECK(conf->readOr("number-etc", 7) == 1); }

  SECTION("readOr reads usr only") { CHECK(conf->readOr("number-usr", 7) == 2); }

  SECTION("readOr reads etc before usr") { CHECK(conf->readOr("number-etc-usr", 7) == 1); }

  SECTION("readOr returns default value without etc and usr") { CHECK(conf->readOr<int>("number", 7) == 7); }
}

} // namespace
