#include "conf.h"
#include "memory.h"

static ds::LogChannel dsLogChannel("dsConf");

namespace ds {

class ConfImpl DS_FINAL : public Conf {
public:
  ConfImpl(System& system) : m_system(system) {}

  boost::optional<JsonFragment> readJsonIfExists(Conf::Location location, gsl::czstring key) DS_OVERRIDE {
    const auto& filename = ds::str(
        [&] {
          switch (location) {
            case Location::SYSCONF:
              return m_system.dirs().sysconfdir;
            case Location::DATA:
              return m_system.dirs().datadir;
          }
          DS_FAIL_REQUIRE("Unknown location", (int)location);
        }(),
        "/ds-conf/", key);
    if (const auto& fileContent = m_system.readFileIfExists(filename)) {
      return ds::fromJson<JsonFragment>(*fileContent);
    }
    return {};
  }

protected:
  LogChannel& logChannel() const DS_OVERRIDE { return dsLogChannel; }

private:
  System& m_system;
};

namespace detail {
LogChannel confLogChannel("dsConf");
} // namespace detail

std::unique_ptr<Conf> makeConf(System& system) { return ds::make_unique<ConfImpl>(system); }

} // namespace ds
