/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <boost/chrono.hpp>
#include <iostream>

namespace ds {

/// Measures elapsed time since the class is created.
///
/// The ostream operator is handy for logging elapsed time.
///
class Elapsed {
public:
  Elapsed() : m_start(boost::chrono::steady_clock::now()) {}

  boost::chrono::steady_clock::duration elapsed() const { return boost::chrono::steady_clock::now() - m_start; }

private:
  boost::chrono::steady_clock::time_point m_start;
};

// Serializes elapsed time.
std::ostream& operator<<(std::ostream& ostream, const Elapsed& x);

} // namespace ds
