/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "exception.h"
#include <ds/catch/catch.h>

namespace {

static const char *TAGS = "[dsException][ds]";

TEST_CASE("dsException", TAGS) {
  SECTION("can be constructed by const char* message") {
    [&] {
      try {
        throw ds::Exception("Problem");
      } catch (const ds::Exception &e) {
        CHECK(e.type() == ds::Exception::Type::FAILED);
        CHECK(e.what() == std::string("Problem"));
        CHECK(e.userMessage() == "");
        return;
      }
      CHECK(false);
    }();
  }

  SECTION("can be constructed by std::string message") {
    [&] {
      try {
        throw ds::Exception(std::string("Problem"));
      } catch (const ds::Exception &e) {
        CHECK(e.type() == ds::Exception::Type::FAILED);
        CHECK(e.what() == std::string("Problem"));
        CHECK(e.userMessage() == "");
        return;
      }
      CHECK(false);
    }();
  }

  SECTION("can be constructed by type and message") {
    [&] {
      try {
        throw ds::Exception(ds::Exception::Type::OVERLOADED, "Problem");
      } catch (const ds::Exception &e) {
        CHECK(e.type() == ds::Exception::Type::OVERLOADED);
        CHECK(e.what() == std::string("Problem"));
        CHECK(e.userMessage() == "");
        return;
      }
      CHECK(false);
    }();
  }

  SECTION("can be constructed by message and userMessage") {
    [&] {
      try {
        throw ds::Exception("Problem", "There was a problem. Please ask support.");
      } catch (const ds::Exception &e) {
        CHECK(e.type() == ds::Exception::Type::FAILED);
        CHECK(e.what() == std::string("Problem"));
        CHECK(e.userMessage() == "There was a problem. Please ask support.");
        return;
      }
      CHECK(false);
    }();
  }

  SECTION("can be constructed by type, message and userMessage") {
    [&] {
      try {
        throw ds::Exception(ds::Exception::Type::OVERLOADED, "Problem", "The system is busy. Please try again later.");
      } catch (const ds::Exception &e) {
        CHECK(e.type() == ds::Exception::Type::OVERLOADED);
        CHECK(e.what() == std::string("Problem"));
        CHECK(e.userMessage() == "The system is busy. Please try again later.");
        return;
      }
      CHECK(false);
    }();
  }

  SECTION("inherits from std::runtime_error") {
    [&] {
      try {
        throw ds::Exception("Problem");
      } catch (const std::runtime_error &e) {
        return;
      }
      CHECK(false);
    }();
  }
}

} // namespace
