/*
    Copyright (c) 2016 digitalSTROM AG, Zurich, Switzerland

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/
#include "exception.h"
#include <ds/enum.h>

namespace ds {

struct Exception::D {
  Type type;
  std::string userMessage;

  DS_MOVABLE_AND_EXPLICIT_COPYABLE(D)
  D(Type type, std::string _userMessage = std::string()) : type(type), userMessage(std::move(_userMessage)) {}
};

Exception::Exception(const std::string& message) : std::runtime_error(message) {}
Exception::Exception(const char* message) : std::runtime_error(message) {}
Exception::Exception(Type type, const std::string& message)
    : std::runtime_error(message), m_maybeD(std::make_shared<D>(type)) {}
Exception::Exception(Type type, const char* message)
    : std::runtime_error(message), m_maybeD(std::make_shared<D>(type)) {}
Exception::Exception(const std::string& message, std::string _userMessage)
    : std::runtime_error(message), m_maybeD(std::make_shared<D>(Type::FAILED, std::move(_userMessage))) {}
Exception::Exception(Type type, const std::string& message, std::string _userMessage)
    : std::runtime_error(message), m_maybeD(std::make_shared<D>(type, std::move(_userMessage))) {}

Exception::Type Exception::type() const { return m_maybeD ? m_maybeD->type : Type::FAILED; };

const std::string& Exception::userMessage() const {
  static std::string empty;
  return m_maybeD ? m_maybeD->userMessage : empty;
};

ds::Enum<Exception::Type> dsEnum(Exception::Type) {
  static constexpr std::pair<Exception::Type, const char*> x[] = {
      {Exception::Type::FAILED, "failed"}, {Exception::Type::OVERLOADED, "overloaded"},
      {Exception::Type::DISCONNECTED, "disconnected"}, {Exception::Type::UNIMPLEMENTED, "unimplemented"},
  };
  return {gsl::make_span(x), "ds::Exception"};
}

std::ostream& operator<<(std::ostream& stream, Exception::Type x) { return ds::ostream(stream, x); }

} // namespace ds
