#include "locale.h"
#include <ds/log.h>
#include "messages.h"

static ds::LogChannel dsLogChannel("dsLocaleLocale");

namespace ds {
namespace locale {

std::string language(const std::locale &x) {
  try {
    return std::use_facet<ds::locale::Messages>(x).language();
  } catch (const std::exception &e) {
    DS_ERROR("Failed to read language from locale.", e);
    return "en_US";
  }
}

std::string sanitizeLanguage(const std::string& lang) {
  // TODO: automatically detect available locales
  static std::vector<std::string> availableLocales = {"so_SO", "da_DK", "de_DE", "en_US", "fr_CH", "it_IT", "nb_NO", "nl_NL", "pl_PL", "pt_PT", "sv_SE", "tr_TR"};
  auto it = std::find_if (availableLocales.begin(), availableLocales.end(), [&](const std::string& s) { return s.compare(lang) == 0; });
  if (it != availableLocales.end()) {
    return *it;
  } else {
    it = std::find_if (availableLocales.begin(), availableLocales.end(), [&](const std::string& s) { return s.compare(0, 2, lang, 0, 2) == 0; });

    if (it != availableLocales.end()) {
      return *it;
    }
  }
  return "en_US";
}

} // namespace locale

namespace detail {
void Str<std::locale>::f(std::ostream &ostream, const std::locale &x, int indent) {
  ostream << (std::has_facet<ds::locale::Messages>(x) ? ::ds::locale::language(x) : x.name());
}

} // namespace detail
} // namespace ds

std::string dsTr(const std::locale &locale, const char *id) { return boost::locale::message(id).str(locale); }
std::string dsTr(const std::locale &locale, const std::string &id) { return boost::locale::message(id).str(locale); }

std::string dsTr(const std::locale &locale, const char *single, const char *plural, int n) {
  return boost::locale::message(single, plural, n).str(locale);
}
std::string dsTr(const std::locale &locale, const std::string &single, const std::string &plural, int n) {
  return boost::locale::message(single, plural, n).str(locale);
}

std::string dsTr(const std::locale &locale, const char *context, const char *id) {
  return boost::locale::message(context, id).str(locale);
}
std::string dsTr(const std::locale &locale, const std::string &context, const std::string &id) {
  return boost::locale::message(context, id).str(locale);
}

std::string dsTr(const std::locale &locale, const char *context, const char *single, const char *plural, int n) {
  return boost::locale::message(context, single, plural, n).str(locale);
}
std::string dsTr(const std::locale &locale, const std::string &context, const std::string &single,
    const std::string &plural, int n) {
  return boost::locale::message(context, single, plural, n).str(locale);
}

std::string dsDtr(const std::locale &locale, const std::string &domain, const char *id) {
  return boost::locale::message(id).str(locale, domain);
}
std::string dsDtr(const std::locale &locale, const std::string &domain, const std::string &id) {
  return boost::locale::message(id).str(locale, domain);
}

std::string dsDtr(const std::locale &locale, const std::string &domain, const char *single, const char *plural, int n) {
  return boost::locale::message(single, plural, n).str(locale, domain);
}
std::string dsDtr(
    const std::locale &locale, const std::string &domain, const std::string &single, const std::string &plural, int n) {
  return boost::locale::message(single, plural, n).str(locale, domain);
}

std::string dsDtr(const std::locale &locale, const std::string &domain, const char *context, const char *id) {
  return boost::locale::message(context, id).str(locale, domain);
}
std::string dsDtr(
    const std::locale &locale, const std::string &domain, const std::string &context, const std::string &id) {
  return boost::locale::message(context, id).str(locale, domain);
}

std::string dsDtr(const std::locale &locale, const std::string &domain, const char *context, const char *single,
    const char *plural, int n) {
  return boost::locale::message(context, single, plural, n).str(locale, domain);
}
std::string dsDtr(const std::locale &locale, const std::string &domain, const std::string &context,
    const std::string &single, const std::string &plural, int n) {
  return boost::locale::message(context, single, plural, n).str(locale, domain);
}
