/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/str.h>
#include <gsl.h>
#include <boost/locale/message.hpp>
#include <locale>

namespace ds {
namespace locale {

/// Returns locale language passed to ds::locale::backend()(language).
/// Throws if the locale was not created by `ds::locale::backend()`.
///
/// We don't use `std::locale::name()`, because
/// `std::locale::name()` can be set only to locales supported
/// by underlaying operating system.
/// We want to be able to carry any language and fail lazily.
///
std::string language(const std::locale &);

std::string sanitizeLanguage(const std::string& lang);

} // namespace locale

namespace detail {

template <>
struct Str<std::locale> {
  static void f(std::ostream &ostream, const std::locale &x, int indent);
};

} // namespace detail
} // namespace ds

// translation functions follow calling convention
// from `boost::locale::message` contructors

/// translate string `id` using `locale` in locale default domain
// `dsTr` is used to harvest translations from source code.
std::string dsTr(const std::locale &locale, gsl::czstring id);
std::string dsTr(const std::locale &locale, const std::string &id);

/// translate plural string `single`, `plural`, `n` using `locale` in locale default domain
std::string dsTr(const std::locale &locale, gsl::czstring single, gsl::czstring plural, int n);
std::string dsTr(const std::locale &locale, const std::string &single, const std::string &plural, int n);

/// translate plural string `id` with `context` using `locale` in locale default domain
std::string dsTr(const std::locale &locale, gsl::czstring context, gsl::czstring id);
std::string dsTr(const std::locale &locale, const std::string &context, const std::string &id);

/// translate plural string `single`, `plural`, `n` with `context` using `locale` in locale default domain
std::string dsTr(const std::locale &locale, gsl::czstring context, gsl::czstring single, gsl::czstring plural, int n);
std::string dsTr(
    const std::locale &locale, const std::string &context, const std::string &single, const std::string &plural, int n);

/// translate string `id` using `locale` in `domain`
std::string dsDtr(const std::locale &locale, const std::string &domain, gsl::czstring id);
std::string dsDtr(const std::locale &locale, const std::string &domain, const std::string &id);

/// translate plural string `single`, `plural`, `n` using `locale` in `domain`
std::string dsDtr(
    const std::locale &locale, const std::string &domain, gsl::czstring single, gsl::czstring plural, int n);
std::string dsDtr(
    const std::locale &locale, const std::string &domain, const std::string &single, const std::string &plural, int n);

/// translate plural string `id` with `context` using `locale` in `domain`
std::string dsDtr(const std::locale &locale, const std::string &domain, gsl::czstring context, gsl::czstring id);
std::string dsDtr(
    const std::locale &locale, const std::string &domain, const std::string &context, const std::string &id);

/// translate plural string `single`, `plural`, `n` with `context` using `locale` in `domain`
std::string dsDtr(const std::locale &locale, const std::string &domain, gsl::czstring context, gsl::czstring single,
    gsl::czstring plural, int n);
std::string dsDtr(const std::locale &locale, const std::string &domain, const std::string &context,
    const std::string &single, const std::string &plural, int n);

// `dsMessage` is used to harvest translations from source code.
typedef boost::locale::message dsMessage;
