/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "reflect.h"
#include <ds/catch/catch.h>

static const char* TAGS = "[dsReflect][ds]";

namespace {

enum class Transport { CAR, PLANE };
struct X {
  int i;
  char c;
  unsigned char uc;
  std::string s;
};
struct Y {
  X x;
  std::vector<X> xs;
  Transport transport;
};

} // namespace

DS_REFLECT(X, (i)(c)(uc)(s));
DS_REFLECT(Y, (x)(xs)(transport));

namespace {

TEST_CASE("isDefaultInstance", TAGS) {
  CHECK(ds::isDefaultInstance(static_cast<const char*>(DS_NULLPTR)));
  CHECK(ds::isDefaultInstance((const char*)""));
  CHECK(!ds::isDefaultInstance("a"));

  CHECK(ds::isDefaultInstance(std::string()));
  CHECK(ds::isDefaultInstance(std::string("")));
  CHECK(!ds::isDefaultInstance(std::string("a")));

  CHECK(ds::isDefaultInstance((char)0));
  CHECK(ds::isDefaultInstance((unsigned char)0));
  CHECK(ds::isDefaultInstance((signed char)0));
  CHECK(ds::isDefaultInstance((unsigned short)0));
  CHECK(ds::isDefaultInstance((signed short)0));
  CHECK(ds::isDefaultInstance(0));
  CHECK(ds::isDefaultInstance(0U));
  CHECK(ds::isDefaultInstance(0LL));
  CHECK(ds::isDefaultInstance(0ULL));
  CHECK(!ds::isDefaultInstance((char)1));
  CHECK(!ds::isDefaultInstance((unsigned char)1));
  CHECK(!ds::isDefaultInstance((signed char)1));
  CHECK(!ds::isDefaultInstance((unsigned short)1));
  CHECK(!ds::isDefaultInstance((signed short)1));
  CHECK(!ds::isDefaultInstance(1));
  CHECK(!ds::isDefaultInstance(1U));
  CHECK(!ds::isDefaultInstance(1LL));
  CHECK(!ds::isDefaultInstance(1ULL));

  CHECK(ds::isDefaultInstance(0.0));
  CHECK(ds::isDefaultInstance(0.0f));
  CHECK(!ds::isDefaultInstance(1.0));
  CHECK(!ds::isDefaultInstance(1.0f));

  CHECK(ds::isDefaultInstance(Transport::CAR));
  CHECK(!ds::isDefaultInstance(Transport::PLANE));

  CHECK(ds::isDefaultInstance(std::vector<int>()));
  CHECK(!ds::isDefaultInstance(std::vector<int>(1, 1)));

  CHECK(ds::isDefaultInstance(X({})));
  CHECK(ds::isDefaultInstance(Y({})));
  {
    X x = {};
    x.i = 1;
    CHECK(!ds::isDefaultInstance(x));
  }
  {
    Y y = {};
    y.x.i = 1;
    CHECK(!ds::isDefaultInstance(y));
  }
}

} // namespace
