#include "system-clock.h"
#include <gsl.h>
#include <sys/time.h>

namespace ds {

SystemClock::~SystemClock() {}

SystemClock::TimePoint SystemClock::now() const {
  return Super::now(); // mockable call to static std method
}

std::string SystemClock::toLocalMsIsoString(TimePoint timePoint) {
  struct tm tm;
  char buf[sizeof "2011-10-08T07:07:09.000+02:00"];
  char fmt[sizeof "2011-10-08T07:07:09.000+02:00"];

  // http://stackoverflow.com/questions/1551597/using-strftime-in-c-how-can-i-format-time-exactly-like-a-unix-timestamp
  auto count = std::chrono::duration_cast<std::chrono::milliseconds>(timePoint.time_since_epoch()).count();
  auto time = gsl::narrow_cast<time_t>(count / 1000);
  auto msec = gsl::narrow_cast<int>(count % 1000);
  ::localtime_r(&time, &tm);
  ::strftime(fmt, sizeof fmt, "%FT%T.%%03d%z", &tm);
  ::snprintf(buf, sizeof buf, fmt, msec);

  /* ensure there is a colon in time zone */
  if (buf[26] != ':') {
    buf[29] = '\0';
    buf[28] = buf[27];
    buf[27] = buf[26];
    buf[26] = ':';
  }
  return std::string(buf);
}

std::string SystemClock::toMsIsoString(TimePoint timePoint) {
  struct tm tm;
  char buf[sizeof "2009-01-02T09:00:00.000Z"];
  char fmt[sizeof "2009-01-02T09:00:00.%03uZ"];

  // http://stackoverflow.com/questions/361363/how-to-measure-time-in-milliseconds-using-ansi-c
  // http://stackoverflow.com/questions/1551597/using-strftime-in-c-how-can-i-format-time-exactly-like-a-unix-timestamp
  auto count = std::chrono::duration_cast<std::chrono::milliseconds>(timePoint.time_since_epoch()).count();
  auto time = gsl::narrow_cast<time_t>(count / 1000);
  auto msec = gsl::narrow_cast<int>(count % 1000);
  ::gmtime_r(&time, &tm);
  ::strftime(fmt, sizeof fmt, "%FT%T.%%03dZ", &tm);
  ::snprintf(buf, sizeof buf, fmt, msec);
  return std::string(buf);
}

SystemClock::SystemClock() {}

SystemClock& systemClock() {
  static SystemClock x;
  return x;
}

} // namespace ds
