/*
 *  Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <ds/common.h>
#include <chrono>

namespace ds {

/// Mockable std::chrono::system_clock
class SystemClock {
public:
  typedef std::chrono::system_clock Super;
  typedef Super::rep rep;
  typedef Super::period period;
  typedef Super::duration duration;
  typedef Super::time_point time_point;
  typedef Super::time_point TimePoint;

  virtual ~SystemClock();

  /// Default implementation calls std::chrono::system_clock::now()
  virtual TimePoint now() const;

  /// Formats TimePoint in ISO8601 format in local time zone with milliseconds precision.
  static std::string toLocalMsIsoString(TimePoint);

  /// Formats TimePoint in ISO8601 format in UTC with milliseconds precision.
  static std::string toMsIsoString(TimePoint);

protected:
  SystemClock();

  friend SystemClock &systemClock();
};

/// returns SystemClock singleton
SystemClock &systemClock();

} // namespace ds

inline std::ostream &operator<<(std::ostream &stream, const ds::SystemClock::TimePoint &tp) {
  stream << ds::SystemClock::toLocalMsIsoString(tp);
  return stream;
}
