#include "system.h"
#include <boost/filesystem.hpp>
#include <cerrno>
#include <fstream>
#include <string>
#include "log.h"
#include "memory.h"

static ds::LogChannel dsLogChannel("dsSystem");
namespace ds {

class SystemImpl DS_FINAL : public System {
public:
  SystemImpl() : m_dirs(Dirs{DS_SYSCONFDIR, DS_DATADIR}) {}
  SystemImpl(Dirs&& dirs) : m_dirs(std::move(dirs)) {}

  const Dirs& dirs() const DS_OVERRIDE { return m_dirs; }

  std::string readFileImpl(const std::string& filename) {
    try {
      std::ifstream in(filename, std::ios::in | std::ios::binary);
      in.exceptions(std::ifstream::failbit | std::ifstream::badbit);
      DS_REQUIRE(in, "");
      in.seekg(0, std::ios::end);
      std::string result;
      result.resize(in.tellg());
      in.seekg(0, std::ios::beg);
      in.read(&result[0], result.size());
      in.close();
      return (result);
    } catch (std::exception &e) {
      DS_FAIL_REQUIRE("Failed to read file", filename, e);
    }
  }

  std::string readFile(const std::string& filename) DS_OVERRIDE {
    DS_DEBUG_ENTER(filename);
    return readFileImpl(filename);
  }

  boost::optional<std::string> readFileIfExists(const std::string& filename) DS_OVERRIDE {
    DS_DEBUG_ENTER(filename);
    if (boost::filesystem::exists(filename)) {
      return readFileImpl(filename);
    }
    return {};
  }

private:
  const Dirs m_dirs;
};

std::unique_ptr<System> makeSystem() { return ds::make_unique<SystemImpl>(); }

std::unique_ptr<System> makeSystem(System::Dirs&& dirs) { return ds::make_unique<SystemImpl>(std::move(dirs)); }

} // namespace ds
