/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DSACTIONREQUEST_H_
#define DSACTIONREQUEST_H_

#include "businterface.h"

#include "dsbusinterfaceobj.h"

namespace dss {

class DSActionRequest : public DSBusInterfaceObj, public ActionRequestInterface {
public:
  DSActionRequest() : DSBusInterfaceObj(), m_pBusEventSink(NULL) {}

  void callScene(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      const uint16_t _scene, const std::string token, const bool _force) DS_OVERRIDE;
  void callSceneMin(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      const uint16_t _scene, const std::string token) DS_OVERRIDE;
  void saveScene(AddressableModelItem *pTarget, const callOrigin_t origin, const uint16_t _scene,
      const std::string token) DS_OVERRIDE;
  void undoScene(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      const uint16_t _scene, const std::string token) DS_OVERRIDE;
  void undoSceneLast(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      const std::string token) DS_OVERRIDE;
  void blink(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      const std::string token) DS_OVERRIDE;
  void setValue(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      const uint8_t value, const std::string token) DS_OVERRIDE;
  void increaseOutputChannelValue(AddressableModelItem *pTarget, const callOrigin_t origin,
      const SceneAccessCategory category, uint8_t channel, const std::string token) DS_OVERRIDE;
  void decreaseOutputChannelValue(AddressableModelItem *pTarget, const callOrigin_t origin,
      const SceneAccessCategory category, uint8_t channel, const std::string token) DS_OVERRIDE;
  void stopOutputChannelValue(AddressableModelItem *pTarget, const callOrigin_t origin,
      const SceneAccessCategory category, uint8_t channel, const std::string token) DS_OVERRIDE;
  void pushSensor(AddressableModelItem *pTarget, const callOrigin_t origin, const SceneAccessCategory category,
      dsuid_t _sourceID, SensorType _sensorType, double _sensorValueFloat, const std::string token) DS_OVERRIDE;
  void muteFire() DS_OVERRIDE;
  void setBusEventSink(BusEventSink *eventSink);

  //< @ret true if locked
  bool isOperationLock(const dsuid_t &dSM, int clusterId) DS_OVERRIDE;

private:
  BusEventSink *m_pBusEventSink;
}; // DSActionRequest

} // namespace dss

#endif
