/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include <dsa-http-client.h>

#include <ds/elapsed.h>
#include <ds/log.h>
#include <ds/memory.h>

static ds::LogChannel dsLogChannel("dsaHttpClient");

namespace dss {

struct DsaHttpResponse {
  bool success;
  std::string message;
  ds::JsonFragment data;

  // to workaround gcc 4.5 compilation errors
  DsaHttpResponse() : success{} {};
  DsaHttpResponse(const DsaHttpResponse&) = delete;
  DsaHttpResponse(DsaHttpResponse&&) = default;
};

class DsaHttpClientImpl DS_FINAL : public DsaHttpClient {
public:
  ds::JsonFragment request(HttpRequest&& x) DS_OVERRIDE {
    DS_DEBUG_ENTER(x);
    x.url = ds::str("http://localhost:8087", x.url);
    HttpClient httpClient;
    std::string httpResponse;
    ds::Elapsed elapsed;
    auto code = httpClient.request(x, &httpResponse);
    DS_NOTICE("Http request", x.url, code, httpResponse, elapsed);
    DS_REQUIRE(code / 100 == 2, "Http request failed.", x.url, code, httpResponse);
    auto response = ds::fromJson<DsaHttpResponse>(httpResponse);
    DS_REQUIRE(response.success, "Dsa request failed.", response);
    return std::move(response).data;
  }
};

std::unique_ptr<DsaHttpClient> makeDsaHttpClient() { return ds::make_unique<DsaHttpClientImpl>(); }

} // namespace dss

DS_REFLECT(dss::DsaHttpResponse, (success)(message)(data));
