/*
    Copyright (c) 2009 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SESSION_H_INCLUDED
#define SESSION_H_INCLUDED

#include <boost/chrono.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/recursive_mutex.hpp>
#include <string>

#include "eventsubscriptionsession.h"
#include "datetools.h"

namespace dss {

  class User;

  class Session {
  public:
    Session(const std::string& _id);
    ~Session();

    void setTimeout(const int _timeout);
    const std::string& getID() const;
    bool isStillValid() const;
    bool isUsed() const;
    void use();
    void unuse();
    void touch();
    void markAsApplicationSession();
    boost::shared_ptr<EventSubscriptionSession> getEventSubscription(const std::string& _token) const;
    void subscribeEventSubscription(EventInterpreter &interp, const std::string& _token, const std::string& _name);
    void unsubscribeEventSubscription(const std::string& _token, const std::string& _name);
    void inheritUserFromSecurity();
    User* getUser() const { return m_pUser; }
    bool isApplicationSession() const { return m_IsApplicationSession; }

  private:
    void deleteEventSubscription(boost::shared_ptr<EventSubscriptionSession> subs);
    boost::shared_ptr<EventSubscriptionSession>
      createEventSubscription(EventInterpreter &interp, const std::string& _token);

  protected:
    mutable boost::mutex m_SubscriptionMutex;
    const std::string m_Token;
    mutable boost::recursive_mutex m_mutex;
    int m_UsageCount;
    boost::chrono::steady_clock::time_point m_lastTouched;
    boost::chrono::seconds m_timeout;
    User* m_pUser;
    bool m_IsApplicationSession;

    typedef std::vector<boost::shared_ptr<EventSubscriptionSession> > subscriptions_t;
    std::vector<boost::shared_ptr<EventSubscriptionSession> > m_subscriptions;
  }; // Session

  std::ostream& operator<<(std::ostream&, const Session&);
}

#endif // defined SESSION_H_INCLUDED
