/*
    Copyright (c) 2017 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "web/websocket/message.h"

#include <boost/algorithm/string/predicate.hpp>

#include <ds/catch/catch.h>

static const char* TAGS = "[dss][websocket][message]";

TEST_CASE("json message parsing", TAGS) {
  SECTION("query") {
    using namespace dss::websocket;
    const std::string jsonMessage = R"__({"id":"1","url":"apartment\/identifyDevices"})__";

    const auto& msg = ds::fromJson<Message>(jsonMessage);
    CHECK(msg.type == MessageType::REQUEST);
    CHECK(msg.url == "apartment/identifyDevices");
    CHECK(msg.id == "1");

    const auto& newJsonMessage = ds::toJson(msg);

    CHECK(newJsonMessage == jsonMessage);
  }

  SECTION("response") {
    using namespace dss::websocket;

    Message msg;
    msg.type = MessageType::REPLY;

    const auto& newJsonMessage = ds::toJson(msg);

    CHECK(boost::algorithm::contains(newJsonMessage, R"("type":"reply")"));
  }
}
