/*
    Copyright (c) 2017 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <cstdint>
#include <list>
#include <string>

#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include <external/civetweb/civetweb.h>

#include "ds/common.h"
#include "dssfwd.h"
#include "web/websocket/connection.h"

namespace dss {

namespace websocket {

///
/// \brief The SimpleServer class is use to provide WebSocket connection.
/// It use civetweb interface and use cbdata pointer to store object pointer (this).
/// This class implement interface and legacy solution for WebSocket.
///
class SimpleServer : private boost::noncopyable {
public:
  SimpleServer(const std::string &url, size_t maxClientCount)
      : m_mgContext(0), m_url(url), m_maxClientCount(maxClientCount), m_trustedPort(0) {}

  virtual ~SimpleServer();

  enum DataHandlerReturn {
    CLOSE_WEBSOCKET = 0,
    KEEP_OPEN_WEBSOCKET = 1,
  };

  void registerToCivetWebServer(mg_context *mgContext, int trustedPort);

  void sendToWebSockets(const Event &e);

  size_t clientCount() const;

  virtual boost::shared_ptr<Connection> createConnectionData(mg_connection *con) const;

protected:
  virtual bool shouldSend(const Event &e, const Connection &c) const;

  virtual void prepareAndSendMsgs(const Event &e, Connection &c) const;

  int connectHandler(const mg_connection *connection, void *cbdata);

  virtual void readyHandler(mg_connection *connection, void *cbdata);

  virtual int dataHandler(mg_connection *connection, int number, char *msg, size_t msgSize, void *cbdata);

  virtual void closeHandler(const mg_connection *connection, void *cbdata);

  static int connectCallback(const mg_connection *connection, void *cbdata);

  static void readyCallback(mg_connection *connection, void *cbdata);

  static int dataCallback(mg_connection *connection, int number, char *msg, size_t msgSize, void *cbdata);

  static void closeCallback(const mg_connection *connection, void *cbdata);

private:
  struct mg_context *m_mgContext;

  mutable boost::mutex m_connectionsMtx;
  std::list<boost::shared_ptr<Connection>> m_connections;

  const std::string m_url;
  size_t m_maxClientCount;
  int m_trustedPort;
};

} // namespace websocket

} // namespace dss
