/*
 *  Copyright (c) 2014 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DS485_DUMMY_INTERFACES__
#define __DS485_DUMMY_INTERFACES__

#include "dss.h"
#include "model/group.h"
#include "messages/vdc-messages.pb.h"
#include "model/modelconst.h"

namespace dss {

class DummyStructureModifyingInterface : public StructureModifyingBusInterface {
public:
  std::string name() const DS_OVERRIDE { return "dummy"; }
  void setZoneID(const dsuid_t& _dsMeterID, const devid_t _deviceID, const int _zoneID) DS_OVERRIDE {}
  void createZone(const dsuid_t& _dsMeterID, const int _zoneID) DS_OVERRIDE {}
  void removeZone(const dsuid_t& _dsMeterID, const int _zoneID) DS_OVERRIDE {}
  void addToGroup(const dsuid_t& _dsMeterID, const int _groupID, const int _deviceID) DS_OVERRIDE {}
  void removeFromGroup(const dsuid_t& _dsMeterID, const int _groupID, const int _deviceID) DS_OVERRIDE {}
  void removeDeviceFromDSMeter(const dsuid_t& _dsMeterID, const int _deviceID) DS_OVERRIDE {}
  void removeDeviceFromDSMeters(const dsuid_t& _deviceDSID) DS_OVERRIDE {}
  void sceneSetName(uint16_t _zoneID, uint8_t _groupID, uint8_t _sceneNumber, const std::string& _name) DS_OVERRIDE {}
  void deviceSetName(dsuid_t _meterDSID, devid_t _deviceID, const std::string& _name) DS_OVERRIDE {}
  void meterSetName(dsuid_t _meterDSID, const std::string& _name) DS_OVERRIDE {}
  void createGroup(uint16_t _zoneID, uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig,
      const std::string& _name) DS_OVERRIDE {}
  void removeGroup(uint16_t _zoneID, uint8_t _groupID) DS_OVERRIDE {}
  void groupSetName(uint16_t _zoneID, uint8_t _groupID, const std::string& _name) DS_OVERRIDE {}
  void createCluster(uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig,
      const std::string& _name) DS_OVERRIDE {}
  void removeCluster(uint8_t _clusterID) DS_OVERRIDE {}
  void groupSetApplication(
      uint16_t _zoneID, uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig) DS_OVERRIDE {}
  void clusterSetName(uint8_t _clusterID, const std::string& _name) DS_OVERRIDE {}
  void clusterSetApplication(
      uint8_t _clusterID, ApplicationType applicationType, uint32_t applicationConfig) DS_OVERRIDE {}
  void clusterSetProperties(
      uint8_t _clusterID, uint16_t _location, uint16_t _floor, uint16_t _protectionClass) DS_OVERRIDE {}
  void clusterSetLockedScenes(uint8_t _clusterID, const std::vector<int>& lockedScenes) DS_OVERRIDE {}
  void clusterSetConfigurationLock(uint8_t _clusterID, bool _lock) DS_OVERRIDE {}
  void clusterSetOperationLock(uint8_t _clusterID, bool _lock) DS_OVERRIDE {}
  void setButtonSetsLocalPriority(const dsuid_t& _dsMeterID, const devid_t _deviceID, bool _setsPriority) DS_OVERRIDE {}
  void setButtonCallsPresent(const dsuid_t& _dsMeterID, const devid_t _deviceID, bool _callsPresent) DS_OVERRIDE {}
  void setZoneHeatingConfig(
      const dsuid_t& _dsMeterID, const uint16_t _ZoneID, const ZoneHeatingConfigSpec_t _spec) DS_OVERRIDE {}
  void setZoneHeatingOperationModes(
      const dsuid_t& _dsMeterID, const uint16_t _ZoneID, const ZoneHeatingOperationModeSpec_t _spec) DS_OVERRIDE {}
  void setZoneSensor(const uint16_t _zoneID, SensorType _sensorType, const dsuid_t& _sensorDSUID) DS_OVERRIDE {}
  void resetZoneSensor(const uint16_t _zoneID, SensorType _sensorType) DS_OVERRIDE {}
  void setCircuitPowerStateConfig(
      const dsuid_t& _dsMeterID, const int _index, const int _setThreshold, const int _resetThreshold) DS_OVERRIDE {}

  void setProperty(const dsuid_t& _meter,
      const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& properties) DS_OVERRIDE {}
  vdcapi::Message getProperty(const dsuid_t& _meter,
      const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& query) DS_OVERRIDE {
    return vdcapi::Message();
  }

  void setFireMuteEnabled(dsuid_t dsm, bool) DS_OVERRIDE {}
  void removeVdc(const dsuid_t& vdsmID, const dsuid_t& vdcID) DS_OVERRIDE {}
};

class DummyStructureQueryBusInterface: public StructureQueryBusInterface {
public:
  virtual std::vector<DSMeterSpec_t> getBusMembers() {
    return std::vector<DSMeterSpec_t>();
  }
  virtual DSMeterSpec_t getDSMeterSpec(const dsuid_t& _dsMeterID) {
    return DSMeterSpec_t();
  }
  virtual std::vector<int> getZones(const dsuid_t& _dsMeterID) {
    return std::vector<int>();
  }
  virtual std::vector<DeviceSpec> getDevicesInZone(const dsuid_t& _dsMeterID, const int _zoneID, bool complete = true) {
    return std::vector<DeviceSpec>();
  }
  virtual std::vector<DeviceSpec> getInactiveDevicesInZone(const dsuid_t& _dsMeterID, const int _zoneID) {
    return std::vector<DeviceSpec>();
  }
  virtual std::vector<GroupSpec_t> getGroups(const dsuid_t& _dsMeterID, const int _zoneID) {
    return std::vector<GroupSpec_t>();
  }
  virtual std::vector<ClusterSpec_t> getClusters(const dsuid_t& _dsMeterID) {
    return std::vector<ClusterSpec_t>();
  }
  virtual std::vector<CircuitPowerStateSpec_t> getPowerStates(const dsuid_t& _dsMeterID) {
    return std::vector<CircuitPowerStateSpec_t>();
  }
  virtual std::vector<std::pair<int,int> > getLastCalledScenes(const dsuid_t& _dsMeterID, const int _zoneID) {
    return std::vector<std::pair<int,int> >();
  }
  virtual std::bitset<7> getZoneStates(const dsuid_t& _dsMeterID, const int _zoneID) {
      return std::bitset<7>(0llu);
  }

  virtual bool getEnergyBorder(const dsuid_t& _dsMeterID, int& _lower, int& _upper) {
    _lower = 0;
    _upper = 0;
    return false;
  }
  virtual DeviceSpec deviceGetSpec(devid_t _id, dsuid_t _dsMeterID) {
    return DeviceSpec();
  }
  virtual std::string getSceneName(dsuid_t _dsMeterID, boost::shared_ptr<Group> _group, const uint8_t _sceneNumber) {
    return "";
  }
  virtual DSMeterHash_t getDSMeterHash(const dsuid_t& _dsMeterID) {
    return DSMeterHash_t();
  }
  virtual void getDSMeterState(const dsuid_t& _dsMeterID, uint8_t *_state) {
    *_state = 0;
  }
  virtual ZoneHeatingConfigSpec_t getZoneHeatingConfig(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingConfigSpec_t();
  }
  virtual ZoneHeatingInternalsSpec_t getZoneHeatingInternals(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingInternalsSpec_t();
  }
  virtual ZoneHeatingStateSpec_t getZoneHeatingState(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingStateSpec_t();
  }
  virtual ZoneHeatingOperationModeSpec_t getZoneHeatingOperationModes(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingOperationModeSpec_t();
  }
  virtual dsuid_t getZoneSensor(const dsuid_t& _meterDSUID, const uint16_t _zoneID, SensorType _sensorType) {
    return DSUID_NULL;
  }
  virtual void getZoneSensorValue(const dsuid_t& _meterDSUID, const uint16_t _zoneID, SensorType _sensorType, uint16_t *_sensorValue, uint32_t *_sensorAge) {
  }
  virtual int getDevicesCountInZone(const dsuid_t& _dsMeterID, const int _zoneID, bool _onlyActive = false) {
    return 0;
  }
  virtual void protobufMessageRequest(const dsuid_t _dSMdSUID, const uint16_t _request_size, const uint8_t *_request, uint16_t *_response_size, uint8_t *_response) {
  }
}; // DummyStructureQueryBusInterface

class DummyBusInterface : public BusInterface {
public:
  DummyBusInterface(StructureModifyingBusInterface* _pStructureModifier,
                    StructureQueryBusInterface* _pStructureQuery,
                    ActionRequestInterface* _pActionRequest)
  : m_pStructureModifier(_pStructureModifier),
    m_pStructureQuery(_pStructureQuery),
    m_pActionRequest(_pActionRequest)
  { }

  DeviceBusInterface* getDeviceBusInterface() DS_OVERRIDE { return NULL; }
  StructureQueryBusInterface* getStructureQueryBusInterface() DS_OVERRIDE { return m_pStructureQuery; }
  MeteringBusInterface* getMeteringBusInterface() DS_OVERRIDE { return NULL; }
  StructureModifyingBusInterface* getStructureModifyingBusInterface() DS_OVERRIDE { return m_pStructureModifier; }
  ActionRequestInterface* getActionRequestInterface() DS_OVERRIDE { return m_pActionRequest; }
  void setBusEventSink(BusEventSink*) DS_OVERRIDE {}
  const std::string getConnectionURI() DS_OVERRIDE { return {}; }

private:
  StructureModifyingBusInterface* m_pStructureModifier;
  StructureQueryBusInterface* m_pStructureQuery;
  ActionRequestInterface* m_pActionRequest;
};

class DummyActionRequestInterface : public ActionRequestInterface {
public:
  void callScene(AddressableModelItem* pTarget, const callOrigin_t _origin, const SceneAccessCategory _category,
      const uint16_t scene, const std::string _token, const bool _force) DS_OVERRIDE {
    Group* pGroup = dynamic_cast<Group*>(pTarget);
    Device* pDevice = dynamic_cast<Device*>(pTarget);
    m_Log += "callScene(";
    if(pGroup != NULL) {
      m_Log += intToString(pGroup->getZoneID()) + "," + intToString(pGroup->getID());
    } else {
      m_Log += intToString(pDevice->getShortAddress());
    }
    m_Log += "," + intToString(scene) + ")";
  }
  void callSceneMin(AddressableModelItem* pTarget, const callOrigin_t _origin, const SceneAccessCategory _category,
      const uint16_t _scene, const std::string _token) DS_OVERRIDE {}
  void saveScene(AddressableModelItem* pTarget, const callOrigin_t _origin, const uint16_t scene,
      const std::string _token) DS_OVERRIDE {}
  void undoScene(AddressableModelItem* pTarget, const callOrigin_t _origin, const SceneAccessCategory _category,
      const uint16_t scene, const std::string _token) DS_OVERRIDE {}
  void undoSceneLast(AddressableModelItem* pTarget, const callOrigin_t _origin, const SceneAccessCategory _category,
      const std::string _token) DS_OVERRIDE {}
  void blink(AddressableModelItem* pTarget, const callOrigin_t _origin, const SceneAccessCategory _category,
      const std::string _token) DS_OVERRIDE {}
  void setValue(AddressableModelItem* pTarget, const callOrigin_t _origin, const SceneAccessCategory _category,
      const uint8_t _value, const std::string _token) DS_OVERRIDE {}
  void increaseOutputChannelValue(
      AddressableModelItem*, const callOrigin_t _origin, SceneAccessCategory, uint8_t, std::string) DS_OVERRIDE {}
  void decreaseOutputChannelValue(
      AddressableModelItem*, const callOrigin_t _origin, SceneAccessCategory, uint8_t, std::string) DS_OVERRIDE {}
  void stopOutputChannelValue(
      AddressableModelItem*, const callOrigin_t _origin, SceneAccessCategory, uint8_t, std::string) DS_OVERRIDE {}
  void pushSensor(AddressableModelItem*, const callOrigin_t _origin, const SceneAccessCategory _category,
      dsuid_t _sourceID, SensorType _sensorType, double _sensorValueFloat, const std::string _token) DS_OVERRIDE {}
  void muteFire() DS_OVERRIDE {}

  bool isOperationLock(const dsuid_t& _dSM, int _clusterId) DS_OVERRIDE { return false; }

  std::string getLog() {
    return m_Log;
  }

  void clearLog() {
    m_Log.clear();
  }
private:
  std::string m_Log;
};


}
#endif
